/*
 * file CcVersion.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cc.CcVersion
 *
 * (C) Copyright IBM Corporation 2004, 2016.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp.cc;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import javax.wvcm.Feedback;
import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.ResourceList;
import javax.wvcm.Task;
import javax.wvcm.Version;
import javax.wvcm.WvcmException;

/**
 * <p>
 * A proxy for a version of a ClearCase element.
 * </p>
 * <p>
 * Each time a new revision of a version-controlled file or 
 * directory is checked in, a new <i>version</i> of that element is created. 
 * Versions are created, in sequence, on the branch of an element selected
 * by the view's config spec. 
 * </p>
 * <p>
 * For more information about version, see the ClearCase "Guide to Developing Software"
 * manual.
 * </p>
 * @see com.ibm.rational.wvcm.stp.cc.CcBranch
 * @see com.ibm.rational.wvcm.stp.cc.CcElement
 * @see com.ibm.rational.wvcm.stp.cc.CcConfigSpec
 * @see com.ibm.rational.wvcm.stp.cc.CcBranchType
 */
public interface CcVersion
    extends Version, CcVobResource 
{    
    /** Flags for the <code>doCcCheckout</code> method. */
    enum CcCheckoutFlag {
        
        /**
         * Indicates whether to checkout this file reserved.
         */
        RESERVED,
        
        /**
         * Fall back to unreserved if a reservation can not be obtained.
         */
        FALLBACK_TO_UNRESERVED,
        
        /**
         * Indicates whether to checkout this file even if the
         * relevant branch is mastered by another replica. The
         * <code>RESERVED</code> flag must NOT be set with this flag.
         * 
         * <p>If the file is mastered by this replica, setting this
         * flag has no effect.
         */
        NON_MASTERED_OK,
        
        /**
         * After a file is 'checkedout', set the file's "last modified" 
         * time to be the time when the version was first created.
         * <p>This only applies to dynamic views.
         */
        PRESERVE_MODIFICATION_TIME;
    }

    
    /** Flags for the doMerge method */
    enum CcMergeFlag {
        /**
         * Informs the merge that there is no data being sent.
         * Just draw the merge arrow.
         * Can not be specified with NO_ARROWS flag.
         */
        NO_DATA,
        /**
         * Requests to only merge the data sent. Do not draw the merge arrow.
         * Can not be specified with NO_DATA flag.
         */
        NO_ARROWS,
        /**
         * If a merge arrow already exists, this will delete and replace that 
         * merge arrow. Does nothing when specified with the NO_ARROWS flag.
         * This flag is currently unsupported for merge in automatic views.
         */
        REPLACE;
    }
    
    /**
     * This version's branch.
     */
    PropertyName<CcBranch> BRANCH =
        new PropertyName<CcBranch>(PROPERTY_NAMESPACE, "version-branch");

    /**
     * Get the value of this version's {@link #BRANCH} property.
     * 
     * @return a proxy for this version's branch
     * @throws WvcmException if property was not requested
     */
    public CcBranch getBranch() throws WvcmException;

    /**
     * This version's element.
     * @see javax.wvcm.Version#VERSION_HISTORY
     */
    PropertyName<CcElement> ELEMENT =
        new PropertyName<CcElement>(PROPERTY_NAMESPACE, "version-element");

    /**
     * Get the value of this version's {@link #ELEMENT} property.
     * 
     * @return a proxy for this version's element
     * @throws WvcmException if property was not requested
     */
    public CcElement getElement() throws WvcmException;

    /**
     * <p>
     * The view-relative path for this version, possibly including
     * the version extension.
     * </p>
     * <p>
     * NOTE: This property is only available if it is retrieved
     * using a method with a view context, such as
     * Resource.doReadProperties(CcView, PropertyRequest).
     * ClearCase needs a view context to resolve version paths.
     * </p>
     */
    PropertyName<String> VIEW_RELATIVE_PATH =
        new PropertyName<String>(PROPERTY_NAMESPACE, "version-view-relative-path");

    /**
     * Get the value of this version's {@link #VIEW_RELATIVE_PATH} property.
     * 
     * @return view-relative path
     * @throws WvcmException
     *     if property was not requested, or if the view context
     *     was not provided
     */
    public String getViewRelativePath() throws WvcmException;
    
    /**
     * <p>
     * The immediate predecessor of this version on this version's branch,
     * or if this is the first version on the branch, the version from
     * which the branch emanates.  Will be <code>null</code> if this version
     * is the <code>/main/0</code> version of its element.
     * </p>
     */
    PropertyName<CcVersion> PREDECESSOR = 
        new PropertyName<CcVersion>(PROPERTY_NAMESPACE, "predecessor");
    
    /**
     * Get the value of this version's {@link #PREDECESSOR} property.
     * @return a CcVersion proxy for this version's predecessor.
     * @throws WvcmException if property was not requested.
     */
    public CcVersion getPredecessor() throws WvcmException;
    
    /**
     * <p>
     * The list of versions that were merged to create this version.
     * This will be empty if this version was not created by a merge
     * operation.
     * </p>
     */
    PropertyName<ResourceList<CcVersion>> MERGE_CONTRIBUTOR_LIST = 
        new PropertyName<ResourceList<CcVersion>>(PROPERTY_NAMESPACE, "merge-contributor-list");

    /**
     * Get the value of this version's {@link #MERGE_CONTRIBUTOR_LIST} property.
     * @return a list of the CcVersion proxies which represent merge contributors for this version.
     * @throws WvcmException if property was not requested.
     */
    public ResourceList<CcVersion> getMergeContributorList() throws WvcmException;
    
    /**
     * The list of tasks associated with this version.
     */
    public PropertyName<ResourceList<Task>> TASK_LIST =
        new PropertyName<ResourceList<Task>>(PROPERTY_NAMESPACE, "version-task-list");
    
    /**
     * Get the value of this versions's (@link #TASK_LIST) property.
     * @return a list of the tasks associated with this version. 
     * @throws WvcmException if property was not requested.
     */
    public ResourceList<Task> getTaskList() throws WvcmException;
    
    /**
     * Set the specified list of tasks as being associated with this version.
     * Overwrites any existing associations.  Can be used to clear all associations
     * by setting an empty list.
     * @param tasks List of tasks to be associated with this version.
     */
    public void setTaskList(ResourceList<Task> tasks) throws WvcmException;
    
    /**
     * Modify the list of tasks associated with this version by adding and
     * removing the items from the specified lists.
     * An intersection between the addition and removal lists is considered an error.
     * @param taskAdditions List of tasks to be added to the list of 
     * associations for this version.  Items in this list which are already associated
     * with the version are ignored.
     * @param taskRemovals List of tasks to be removed from the list of 
     * associations for this version.  Items in this list which are not associated
     * with the version are ignored.
     */
    public void setTaskList(ResourceList<Task> taskAdditions, ResourceList<Task> taskRemovals) throws WvcmException;

    /**
     * Add the specified label to the version.
     * @param label Label to be applied
     * @param view View context
     * @throws WvcmException
     * @see javax.wvcm.Version#doAddLabel(java.lang.String, javax.wvcm.Feedback)
     */
    public Version doAddLabel(String label, CcView view, Feedback feedback) throws WvcmException;    
    
    /**
     * Add the specified label to the version.
     * @param comment The comment for this operation, or null for no comment
     * @param label Label to be applied
     * @param view View context
     * @throws WvcmException
     * @see javax.wvcm.Version#doAddLabel(java.lang.String, javax.wvcm.Feedback)
     */
    public Version doAddLabel(String comment, String label, CcView view, Feedback feedback) throws WvcmException;

    /**
     * Set the specified label on the version.
     * @param label Label to be applied
     * @param view View context
     * @throws WvcmException
     * @see javax.wvcm.Version#doSetLabel(java.lang.String, javax.wvcm.Feedback)
     */
    public Version doSetLabel(String label, CcView view, Feedback feedback) throws WvcmException;    
    
    
    /**
     * Set the specified label on the version.
     * @param comment The comment for this operation, or null for no comment
     * @param label Label to be applied
     * @param view View context
     * @throws WvcmException
     * @see javax.wvcm.Version#doSetLabel(java.lang.String, javax.wvcm.Feedback)
     */
    public Version doSetLabel(String comment, String label, CcView view, Feedback feedback) throws WvcmException;
    
    /**
     * Remove the specified label from the version.
     * @param label Label to be removed
     * @param view View context
     * @throws WvcmException
     * @see javax.wvcm.Version#doRemoveLabel(java.lang.String, javax.wvcm.Feedback)
     */
    public Version doRemoveLabel(String label, CcView view, Feedback feedback) throws WvcmException;

    /**
     * Create a Merge hyperlink pointing from this version to the specified destination version.
     * @param toVersion Destination version for the hyperlink
     * @return A new proxy for this version, whose properties are specified by feedback.
     */
    public CcVersion doCreateMergeArrow(CcVersion toVersion, Feedback feedback) throws WvcmException;
    
    /**
     * Create a Merge hyperlink pointing from this version to the specified destination version.
     * @param toVersion Destination version for the hyperlink
     * @param view View context (ensures path EVs set for mkhlink when a merge is being made).
     * @return A new proxy for this version, whose properties are specified by feedback.
     */
    public CcVersion doCreateMergeArrow(CcVersion toVersion, CcView view, Feedback feedback) throws WvcmException;
    
    /**
     * <p>Check out the specific version of a file or directory specified
     * by this proxy.  The resource is checked out to the specified ClearCase view.
     * Note that directory version checkout is only supported for dynamic views.
     * </p>
     * <p>If the view is a UCM view, the caller must insure there is a
     * {@link javax.wvcm.Workspace#CURRENT_ACTIVITY} for this operation.
     * The checked out file will be added to the current activity's change set.
     * The caller may explicitly specify an activity using this resource's
     * {@link javax.wvcm.ControllableResource#setActivity} method.  In that case,
     * the specified activity will become the new current activity.
     * Otherwise, the existing current activity will be used.
     * If the view is a UCM view and there is no current activity, the operation
     * will fail.
     * </p>
     * <p>The caller may optionally specify a checkout comment using this
     * resource's {@link javax.wvcm.Resource#setComment} method.
     * </p>
     * @param flags array of flags which specify the behavior of the operation
     * @param view View context
     * @param feedback 
     * @return new CcFile proxy representing the checked out file in the specified view
     * @throws WvcmException
     */    
    CcFile doCcCheckout(CcCheckoutFlag[] flags, CcView view, Feedback feedback)
        throws WvcmException;

    
}
